/*
 * "ESWIN Computing" Memory Copy SDK
 *
 * Copyright 2024, Beijing ESWIN Computing Co., Ltd.. All rights reserved.
 *
 * Unauthorized copying, modification, or distribution of this program,
 * in whole or in part, is strictly prohibited without prior written
 * permission from Beijing ESWIN Computing Technology Co., Ltd.
 *
 * Authors: Yuyang Cong <congyuyang@eswincomputing.com>
 */

#ifndef __ES_MEMCP_H__
#define __ES_MEMCP_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "es_common.h"
#include "es_errno.h"
#include "es_type.h"
#include <linux/dma_memcp.h>
#include <pthread.h>

#define ES_MEMCP_ERROR_BASE               ES_DEF_ERR(ES_ID_MEMCP, EN_ERR_LEVEL_ERROR, 0)
#define ES_MEMCP_ERROR_NUM(X)             (ES_MEMCP_ERROR_BASE + X)

#define ES_MEMCP_ERROR_NULL_PTR  ES_MEMCP_ERROR_NUM(9)
#define ES_MEMCP_BUSY            ES_MEMCP_ERROR_NUM(10)
#define ES_MEMCP_INVALID_HANDLE  ES_MEMCP_ERROR_NUM(11)
#define ES_MEMCP_MEMORY_ERROR    ES_MEMCP_ERROR_NUM(12)
#define ES_MEMCP_OPERATION_FAIL  ES_MEMCP_ERROR_NUM(13)
#define ES_MEMCP_UNKNOWN_CMD     ES_MEMCP_ERROR_NUM(14)
#define ES_MEMCP_SYNC_ERROR      ES_MEMCP_ERROR_NUM(15)


#define ES_MEMCP_DEV_NOT_FOUND            ES_MEMCP_ERROR_NUM(16)
#define ES_MEMCP_IOCTL_FAIL               ES_MEMCP_ERROR_NUM(17)
#define ES_MEMCP_MEM_ALLOC_FAIL           ES_MEMCP_ERROR_NUM(18)

/**
 * @brief Creates a command queue.
 *
 * This function initializes and creates a new command queue for managing asynchronous
 * operations. The handle to the newly created command queue is returned in the output parameter.
 *
 * @param[out] cmdQHandle Pointer to a variable where the handle to the created command queue will be stored.
 *
 * @return ES_S32 Returns ES_SUCCESS on success, or an error code on failure.
 */
ES_S32  ES_CmdQueue_Create(ES_S32 *cmdQHandle);

/**
 * @brief Destroys an existing command queue.
 *
 * This function releases the resources associated with a command queue. Any commands
 * that are still pending in the queue will be completed before the queue is destroyed.
 *
 * @param cmdQHandle Handle to the command queue to be destroyed.
 *
 * @return ES_S32 Returns ES_SUCCESS on success, or an error code on failure.
 */
ES_S32  ES_CmdQueue_Destroy(ES_S32 cmdQHandle);

/**
 * @brief Waits for all operations in the command queue to complete.
 *
 * This function blocks the calling thread until all operations in the specified
 * command queue are completed. It ensures that any pending asynchronous commands
 * are finished before proceeding.
 *
 * @param cmdQHandle Handle to the command queue to synchronize.
 *
 * @return ES_S32 Returns ES_SUCCESS on success, or an error code on failure.
 */
ES_S32  ES_CmdQueue_Sync(ES_S32 cmdQHandle);

/**
 * @brief Queries the status of a command queue.
 *
 * This function checks whether all operations in the specified command queue have
 * completed. If the queue is busy, it will return an appropriate status code.
 *
 * @param cmdQHandle Handle to the command queue to query.
 *
 * @return ES_S32 Returns ES_SUCCESS if the queue is idle, or an error code if the queue
 *         is still executing commands. Other error codes may be returned on failure.
 */
ES_S32  ES_CmdQueue_Query(ES_S32 cmdQHandle, struct esw_cmdq_query *query);

/**
 * @brief Initiates an asynchronous memory copy operation.
 *
 * This function enqueues a memory copy command into the specified command queue,
 * allowing the calling thread to continue execution without waiting for the copy
 * operation to complete. The actual copy will be performed asynchronously.
 *
 * @param cmdQHandle  Command queue handle where the copy command will be enqueued.
 * @param srcFd       Source file descriptor for the DMA buffer.
 * @param srcOffset   Offset within the source buffer to start copying from.
 * @param dstFd       Destination file descriptor for the DMA buffer.
 * @param dstOffset   Offset within the destination buffer to start copying to.
 * @param len         Length of data to copy, in bytes.
 * @param timeout     Timeout for the memory copy operation, in milliseconds.
 *
 * @return ES_S32     Returns ES_SUCCESS on successful enqueue, or an error code on failure.
 */
ES_S32 ES_Memcpy_Async(ES_S32 cmdQHandle, ES_U64 srcFd, ES_U64 srcOffset,
                 ES_U64 dstFd, ES_U64 dstOffset, ES_U64 len, ES_S32 timeout);

#ifdef __cplusplus
}
#endif

#endif  // ES_MEMCP_H